import {
  Component,
  Input,
  Output,
  ElementRef,
  ViewChild,
  ViewEncapsulation,
  EventEmitter,
  OnInit
} from '@angular/core'
import * as anim from 'css-animation'
import { BooleanFieldValue } from '../../utils'

@Component({
  selector: 'Alert',
  encapsulation: ViewEncapsulation.None,
  templateUrl: './template.html',
  styleUrls: [
    './style.less'
  ]
})
export class Alert implements OnInit {
  @Input() type: 'success' | 'info' | 'warning' | 'error'
  @Input() @BooleanFieldValue() closable: boolean
  @Input() closeText: string // TODO: support Element
  @Input() message: string // TODO: support Element
  @Input() description: string // TODO: support Element
  @Input() showIcon: boolean
  @Input() banner: boolean
  @Output() onClose: EventEmitter<MouseEvent> = new EventEmitter<MouseEvent>()
  @ViewChild('animateEle') animateEle: ElementRef

  closing = true
  closed = false
  alertClass = {}
  iconType = ''
  prefixCls = 'ant-alert'

  handleClose(e: MouseEvent) {
    e.preventDefault()

    this.closing = false
    this.onClose.emit(e)

    anim(this.animateEle.nativeElement, `${this.prefixCls}-slide-up-leave`, () => {
      this.closed = true
      this.closing = true
      this.render()
    })
  }

  private render() {
    this.showIcon = this.banner && this.showIcon === undefined ? true : this.showIcon
    this.type = this.banner && this.type === undefined ? 'warning' : this.type || 'info'

    let iconType = ''
    switch (this.type) {
      case 'success':
        iconType = 'check-circle'
        break
      case 'info':
        iconType = 'info-circle'
        break
      case 'error':
        iconType = 'cross-circle'
        break
      case 'warning':
        iconType = 'exclamation-circle'
        break
      default:
        iconType = 'default'
    }

    if (!!this.description) {
      iconType += '-o'
    }
    this.iconType = iconType

    const prefixCls = this.prefixCls
    this.alertClass = {
      'ant-alert': true,
      [`${prefixCls}-${this.type}`]: true,
      [`${prefixCls}-close`]: !this.closing,
      [`${prefixCls}-with-description`]: !!this.description,
      [`${prefixCls}-no-icon`]: !this.showIcon,
      [`${prefixCls}-banner`]: !!this.banner
    }

    // closeable when closeText is assigned
    if (this.closeText) {
      this.closable = true
    }
  }

  ngOnInit() {
    this.render()
  }
}
